from .wx import (
    WeChat,
    Chat
)
from .param import WxParam
import pythoncom

pythoncom.CoInitialize()

__version__ = "0.2.1"
__author__ = "laychou666"

__all__ = [
    'WeChat',
    'Chat',
    'WxParam'
]