# setup.py
from setuptools import setup, find_packages
from pathlib import Path

# 读取 README.md 作为长描述
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text(encoding='utf-8')

setup(
    name="wxautoz",
    version="0.2.2",
    author="laychou666",
    author_email="laychou666@gmail.com",
    maintainer="laychou666",
    maintainer_email="laychou666@gmail.com",
    description="基于 wxauto (by cluic) 的增强版本，添加了后台调度功能",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/laychou666/wxautoz",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: Microsoft :: Windows",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.8",
    install_requires=[
        "pywin32",
        "uiautomation",
        "colorama",
        "comtypes",
        "pyperclip",
        "psutil",
        "Pillow",
    ],
    keywords="wechat automation wxauto background",
    project_urls={
        "Bug Reports": "https://github.com/laychou666/wxautoz/issues",
        "Source": "https://github.com/laychou666/wxautoz",
        "Original Project": "https://github.com/cluic/wxauto",
    },
    license="Apache License 2.0",
)